open Linear_algebra

(** Guess for Hartree-Fock calculations.
Ref: https://pubs.acs.org/doi/10.1021/acs.jctc.8b01089
*)

type ao = Ao.Ao_dim.t
type mo = Mo_dim.t

type guess =
| Hcore  of (ao,ao) Matrix.t (* Core Hamiltonian Matrix *)
| GWH    of (ao,ao) Matrix.t (* Generalized Wolfsberg-Helmholtz (GWH) *)
| Huckel of (ao,ao) Matrix.t (* Huckel Hamiltonian Matrix *)
| Matrix of (ao,mo) Matrix.t (* Guess Eigenvectors *)

type t = guess


val make :
  ?nocc:int ->
  guess:[ `Hcore | `GWH | `Huckel | `Matrix of (ao,mo) Matrix.t ] ->
  Ao.Basis.t -> t

